# Copyright (C) 2004  Alex Schroeder <alex@emacswiki.org>
#           (C) 2005  Fletcher T. Penney <http://fletcher.freeshell.org/>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#    Free Software Foundation, Inc.
#    59 Temple Place, Suite 330
#    Boston, MA 02111-1307 USA

$ModulesDescription .= '<p>$Id: atom.pl,v 1.8alt 2004/10/10 15:14:44 fletcherpenney Exp $</p>';

$Action{atom} = \&DoAtom;

$cgiParams = "?action=atom;";

sub DoAtom {
  print GetHttpHeader('application/xml');
  DoRc(\&GetRcAtom);
}

sub AtomTime {
  my ($sec, $min, $hour, $mday, $mon, $year) = gmtime(shift);
  return sprintf( "%4d-%02d-%02dT%02d:%02d:%02dZ",
		  $year+1900, $mon+1, $mday, $hour, $min, $sec);
}

sub AtomTag {
  my ($tag, $value) = @_;
  return '' unless $value;
  return "<$tag>$value</$tag>\n";
}

sub AtomTagWithType {
  my ($tag, $value, $escaped) = @_;
  return '' unless $value;
  if ($escaped) {
    return "<$tag type=\"html\">$value</$tag>\n";
  } else {
    return "<$tag type=\"text\">$value</$tag>\n";
  }
}

sub AtomID {
	my ($url, $date) = @_;

	my ($sec, $min, $hour, $mday, $mon, $year) = gmtime($date);
	my $tagDate = sprintf( "%4d-%02d-%02d", $year+1900, $mon+1, $mday);
	
	# This can be used to generate a tag: rather than http: 
	# for the id
	# see http://diveintomark.org/archives/2004/05/28/howto-atom-id
	# but I am holding off for now
	
	#$url =~ s/^http:\/\//tag:/i;
	#$url =~ s/(.*?)\//$1,$tagDate:\//;

	return $url;
}

sub GetRcAtom {
  return if $CollectingJournal; # avoid infinite loops
  local $CollectingJournal = 1;
  # from http://www.ietf.org/internet-drafts/draft-ietf-atompub-format-01.txt
  print <<EOT;
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<?xml-stylesheet href="http://www.blogger.com/styles/atom.css" type="text/css"?>
<feed xmlns="http://www.w3.org/2005/Atom">
EOT
  my $title = $SiteName;
  my $link = $ScriptName;

  if (GetParam('full', 0)) {
  	$cgiParams .= "full=1;";
  }
  print AtomTagWithType('title', QuoteHtml($title), 1);
  print "<link href=\"$link/$cgiParams\" rel=\"self\" title=\"$title\" type=\"application/atom+xml\"/>\n";
  print "<link href=\"$link\" rel=\"alternate\" title=\"$title\" type=\"text/html\"/>\n";
  print AtomTag('author', AtomTag('name', $RssPublisher)) if $RssPublisher;
  print AtomTag('contributor', AtomTag('name', $RssContributor)) if $RssContributor;
  print "<generator uri=\"http://www.oddmuse.org/\">Oddmuse</generator>\n";
  print AtomTag('id',$link);
  print AtomTagWithType('rights', QuoteHtml($RssRights), 1) if $RssRights;
  print <<EOT;
EOT
  print AtomTag('updated', AtomTime($LastUpdate));
  my @excluded = ();
  if (GetParam('exclude', 1)) {
    foreach (split(/\n/, GetPageContent($RssExclude))) {
      if (/^ ([^ ]+)[ \t]*$/) {  # only read lines with one word after one space
	push(@excluded, $1);
      }
    }
  }
  GetRc(sub {},
	sub {
	  my ($pagename, $timestamp, $host, $userName, $summary, $minor, $revision, $languages, $cluster) = @_;
	  return if grep(/$pagename/, @excluded);
	  my $title = FreeToNormal($pagename);
	  $title =~ s/_/ /g;
	  my $link = $ScriptName . ($UsePathInfo ? '/' : '?') . UrlEncode($pagename);
	  my $author = $userName;
	  $author = $host unless $author;
	  # output
	  print "<entry>\n",
	    "<id>" . AtomID($link) . "</id>\n",
	    AtomTagWithType('title', QuoteHtml($title), 1),
	    "<link href=\"$link\" rel=\"self\" title=\"$title\" type=\"text/html\"/>\n",
	    AtomTag('author', AtomTag('name', $author)),
	    AtomTag('updated', AtomTime($timestamp)),
	    AtomTag('published', AtomTime($timestamp)),
	    AtomTagWithType('summary', QuoteHtml($summary), 1);
	  if (GetParam('full', 0)) {
	    print '<content type="xhtml">', "\n",
	      '<div xmlns="http://www.w3.org/1999/xhtml">', "\n";
	    OpenPage($pagename);
	    PrintPageDiff();
	    PrintPageHtml();
	    print "\n</div>\n</content>\n";
	  } else {
	  	print "<content type=\"xhtml\"></content>\n";
	  }
	  print "</entry>\n";
	},
	@_);
  print "</feed>\n";
  return '';
}
