# Copyright (C) 2003, 2004  Brock Wilcox <awwaiid@thelackthereof.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#    Free Software Foundation, Inc.
#    59 Temple Place, Suite 330
#    Boston, MA 02111-1307 USA

use Digest::MD5 qw(md5 md5_hex md5_base64);

use vars qw($GraphDir $GraphLinkDir $GraphExtendPath);

$ModulesDescription .= '<p>Graph</p>';

$GraphExtendPath = ':/usr/bin:/bin:/usr/local/bin';

$GraphDir    = "/home/services/lighttpd/html/graph";
$GraphLinkDir= "http://czarny.is-a-geek.net/static/graph";

push(@MyRules, \&GraphRule);

sub GraphRule {
  if (m/\G&lt;graph(.*?)&gt;(.*?)&lt;\/graph&gt;/gcms) {
    return &MakeGraph($2,$1);
  }
  return undef;
}

sub MakeGraph {
  my ($graph, $ops) = @_;
  $ENV{PATH} .= $GraphExtendPath
    if $GraphExtendPath and $ENV{PATH} !~ /$GraphExtendPath/;
  $graph = UnquoteHtml($graph); # Change &lt; back to <, for example
  my $hash = md5_hex($graph);
  $hash =~ s/%//g;
  # check cache
  if (-f "$GraphDir/$hash.png"
      and not -z "$GraphDir/$hash.png") {
    return ("<img class=\"graph\" src=\"$GraphLinkDir/$hash.png\" alt=\"Graph\"$ops />");
  }
  # read template and replace <math>
  mkdir($GraphDir) unless -d $GraphDir;

  #setup rendering directory
  my $dir = "$GraphDir/$hash";
  if (-d $dir) {
    unlink (glob('$dir/*'));
  } else {
    mkdir($dir) or return "[Unable to create $dir]";
  }
  chdir ($dir) or return "[Unable to switch to $dir]";
  WriteStringToFile ("graph.txt", $graph);
  qx( dot -Tpng graph.txt > out.png );
  return "[Dot error: $!]" if $!;
  my $result;
  if (-f 'out.png' and not -z 'out.png') {
    my $png = ReadFileOrDie("out.png");
    WriteStringToFile ("$GraphDir/$hash.png", $png);
    $result = "<img class=\"graph\" src=\"$GraphLinkDir/$hash.png\" alt=\"Graph\"/>";
  } else {
    $result = "[Error retrieving graph]";
  }
  unlink (glob('*'));
  chdir ($GraphDir);
  rmdir ($dir);
  return $result;
}

